//Grigorescu Ana-Maria
//332 CC
//2022

//Needed libraries
#include "LedControl.h"
#include <LiquidCrystal_I2C.h>

// Matrix pins
int DIN = 11;
int CS = 10;
int CLK = 13;

// Matrix needed info
int numDevices = 1; // number of max7219 matrixes connected
// Possible positions for the columns of the matrix
int spectralHeight[] = {0b00000000,0b10000000,0b11000000,
                        0b11100000,0b11110000,0b11111000,
                        0b11111100,0b11111110,0b11111111};
//Initialization of the matrix module
LedControl lc=LedControl(DIN, CLK, CS, numDevices);

// selecting the analog input pin
const int inputPin = A1;
// size of the window in which we read and interpret the sound spectrum
const int inputWindow = 200;
// a sound measurement
unsigned int inputSample;

// values choosen after reading multiple datasheets wannabe 
//of the microphone module I found
const int sensitivity = -58;     // microphone sensitivity in dB
const int gain = 20;             // op-amp gain dB

//index we're going to use during the loop in order to cycle 
//through the columns of the matrix
int idx = 0;

//LCD initialization:
LiquidCrystal_I2C lcd(0x27,16,2);

// button info: 
int buttonPin = 2;
int buttonStatus = 0;
long lastDebounceTime = 0;
long debounceDelay = 500;

void setup() {
  // initializing the analog input
  lc.shutdown(0, false);
  lc.setIntensity(0, 1);
  lc.clearDisplay(0);
  
  // initializing the serial communication
//  Serial.begin(9600);

  //initializing LCD
  lcd.init();
  lcd.clear();
  lcd.backlight();

  //initializing button
  // we use input pullup in order to not use
  // a resistor
  // https://docs.arduino.cc/tutorials/generic/digital-input-pullup
  pinMode(2, INPUT_PULLUP);
  
}

void loop() {
//------------------------------------------------------------------------------------- Variable declaration
  // two variables for minimum and maximum values in window
  unsigned int avgSound = 0;
  unsigned int inputMax = 0;
  unsigned int inputMin = 1024;
  unsigned int peakToPeak = 0;         // peak-to-peak level

//------------------------------------------------------------------------------------ Read button + treat debounce
  if ((millis() - lastDebounceTime) > debounceDelay && digitalRead(buttonPin) == LOW) {
      if (buttonStatus == 1) {
          buttonStatus = 0;
          lastDebounceTime = millis();
      } else {
          buttonStatus = 1;
          lastDebounceTime = millis();
        } 
  }
//------------------------------------------------------------------------------------ Get sound sample
  // loop for the window
  for (unsigned int i = 0; i < inputWindow; i++) {
    // read in a single value
    inputSample = analogRead(inputPin);
    // get the minimum and maximum value
    inputMin = min(inputMin, inputSample);
    inputMax = max(inputMax, inputSample);
    avgSound += inputSample;
  }

  avgSound = avgSound / inputWindow;
//------------------------------------------------------------------------------------ dB meter
  peakToPeak = inputMax - inputMin; // max - min = peak-peak amplitude

  double volts = ((peakToPeak * 5.0) / 1024)*10;    // convert to volts,gain 20db  = 10V                                                 
  
  // https://electronics.stackexchange.com/questions/96205/how-to-convert-volts-in-db-spl
  // conversion using http://www.sengpielaudio.com/calculator-gainloss.htm
   double volts_db = 20*log10(volts/0.001259);                                        
    
     //  94 is a 1 Pa expressed as dB SPL                            
   double spl_db = volts_db + 94 + sensitivity - gain; 

//------------------------------------------------------------------------------------- change Matrix + LCD

  displayLed(avgSound, idx);
  idx++;

  if (buttonStatus == 0) {
          lcd.clear();
          lcd.setCursor(2,0);
          lcd.print("dB value:");
          lcd.setCursor(2,1);
          lcd.print(String(volts_db));
  } else {
      lcd.clear();
      lcd.setCursor(2,0);
      lcd.print("spl dB value:");
      lcd.setCursor(2,1);
      lcd.print(String(spl_db));
    }    

// reset counter for looping through the colums on the matrix
  if (idx == 8) {
      idx = 0;
  }  
}

//Ma gandeam ca as putea sa fac ceva destept cu variabila asta, inca nu mi-a venit ideea
int avgSoundMem = 0;
// variables used towards randomizing some effects on the matrix
int randomizeTime = 5;
int counter = 0;

void displayLed(int soundLVL, int col) {
  if (counter == randomizeTime) {
      randomizeTime = random(0,5);
      if (soundLVL > 25) {
          lc.setColumn(0, random(0,8), spectralHeight[random(0,8)]);
          lc.setColumn(0, random(0,8), spectralHeight[random(0,8)]);
        }
      
      counter = 0;
    }
    
  counter++;

  // after setting the potentiometer of the microphone module
  //and doing a lot of tests, i have found these values
  //work pretty well for the light effects
  if (soundLVL < 25) {
    lc.setColumn(0, col, spectralHeight[0]);
  } else if (soundLVL < 70) {
      lc.setColumn(0, col, spectralHeight[1]);
    } else if (soundLVL < 150) {
      lc.setColumn(0, col, spectralHeight[2]);
    } else if (soundLVL < 210) {
      lc.setColumn(0, col, spectralHeight[3]);
    } else if (soundLVL < 260) {
      lc.setColumn(0, col, spectralHeight[4]);
    } else if (soundLVL < 300) {
      lc.setColumn(0, col, spectralHeight[5]);
    } else if (soundLVL < 350) {
      lc.setColumn(0, col, spectralHeight[6]);
    } else if (soundLVL > 350) {
      lc.setColumn(0, col, spectralHeight[7]);
    }

    avgSoundMem = soundLVL;
}
